/* GUTS - gUnrealTools
 * Copyright (C)2005 Jeffrey Bakker

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ___________________________________ .. .
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <glib/gstdio.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"


GtkWidget	*UModWnd;
GtkWidget	*CacheWnd;
GtkWidget	*CompressWnd;
GtkWidget	*ServerWnd;
GtkWidget	*AboutWnd;

gchar		*mut;
gchar		*map;

gchar		*GameType;
gchar		*MapFilter;
gint		GoalScore	= 25;
gint		TimeLimit	= 0;
gboolean	WeaponStay	= FALSE;
gboolean	UseTeamSkins	= FALSE;
gboolean	BehindView	= FALSE;
gboolean	WeaponThrow	= FALSE;
gboolean	Translocator	= FALSE;
gboolean	ForceRespawn	= FALSE;
gboolean	MustBeReady	= FALSE;
gchar		*AdminEmail	= "";
gchar		*AdminName	= "admin";
gchar		*AdminPass	= "password";
gchar		*GamePass	= "";
gchar		*ServerName	= "GUTS Linux Server";
gchar		*ServerMOTD	= "GUTS Linux Server|http://gunrealtools.sf.net";
gint		MaxPlayers	= 8;
gint		MaxSpectators	= 1;
gint		MinNetPlayers	= 1;
gboolean	StatsLogging	= FALSE;


gchar		*UModFile = "";
gchar		*UTDir = "";


/*** Construct GtkTreeViews ***************************************************/
static GtkTreeModel *
create_fill_cachelist(void)
{
	GtkListStore	*store;
	store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);

	return GTK_TREE_MODEL(store);
}


GtkTreeModel*
create_model_cachelist(GtkWidget *TVBL)
{
	GtkCellRenderer		*renderer;
	GtkTreeModel		*model;
	GtkWidget		*view;

	view = TVBL;
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
							-1,
							"Filename",
							renderer,
							"text", 0,
							NULL);

	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
							-1,
							"Cache Filename (MD5 Hash)",
							renderer,
							"text", 1,
							NULL);

	model = create_fill_cachelist();

	gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
	g_object_unref(model);
	
	return model;
}

static GtkTreeModel *
create_fill_mutatorlist(void)
{
	GtkListStore	*store;
	store = gtk_list_store_new(1, G_TYPE_STRING);

	return GTK_TREE_MODEL(store);
}


GtkTreeModel*
create_model_mutatorlist(GtkWidget *TVBL)
{
	GtkCellRenderer		*renderer;
	GtkTreeModel		*model;
	GtkWidget		*view;

	view = TVBL;
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
							-1,
							"Enabled Mutators",
							renderer,
							"text", 0,
							NULL);

	model = create_fill_mutatorlist();

	gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
	g_object_unref(model);
	
	return model;
}

static GtkTreeModel *
create_fill_umodlist(void)
{
	GtkListStore	*store;
	store = gtk_list_store_new(1, G_TYPE_STRING);

	return GTK_TREE_MODEL(store);
}


GtkTreeModel*
create_model_umodlist(GtkWidget *TVBL)
{
	GtkCellRenderer		*renderer;
	GtkTreeModel		*model;
	GtkWidget		*view;

	view = TVBL;
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
							-1,
							"Files to Unpack",
							renderer,
							"text", 0,
							NULL);

	model = create_fill_umodlist();

	gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
	g_object_unref(model);
	
	return model;
}

static GtkTreeModel *
create_fill_compresslist(void)
{
	GtkListStore	*store;
	store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_ULONG);

	return GTK_TREE_MODEL(store);
}


GtkTreeModel*
create_model_compresslist(GtkWidget *TVBL)
{
	GtkCellRenderer		*renderer;
	GtkTreeModel		*model;
	GtkWidget		*view;

	view = TVBL;
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
							-1,
							"Files to Compress/Decompress",
							renderer,
							"text", 0,
							NULL);

	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),
							-1,
							"Size (bytes)",
							renderer,
							"text", 1,
							NULL);

	model = create_fill_compresslist();

	gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
	g_object_unref(model);
	
	return model;
}

void
get_utinstalldir()
{
	FILE	*GUTSRC;
	gchar	*RcFile;
	gchar	*MakeRc;
	gchar	buffer;

	if(strcmp(UTDir, "") != 0)
	{
		return;	
	}
	RcFile = g_strdup_printf("%s/.gutsrc", getenv("HOME"));

	if((GUTSRC = fopen(RcFile, "r")) == NULL)
	{
		/* TODO: display error message */
		gtk_main_quit();
	}

	while((buffer = getc(GUTSRC)) != EOF)
	{
		UTDir = g_strdup_printf("%s%c", UTDir, buffer);
	}
	fclose(GUTSRC);
}


/*** Main Window **************************************************************/
void
on_btnMainCache_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	gint	lnum;
	gchar	buffer;
	gchar	*listfile;
	gchar	*lcurfile;
	FILE	*FileList;

	GtkListStore	*lsCacheList;
	GtkTreeIter	iter;

	get_utinstalldir();
	CacheWnd = create_CacheManWnd();
	lsCacheList = GTK_LIST_STORE( create_model_cachelist(lookup_widget( GTK_WIDGET(CacheWnd), "tvCacheFiles") ) );

	listfile = g_strdup_printf("%s/.ut2004/Cache/cache.ini", getenv("HOME"));
	
	if((FileList = fopen(listfile, "r")) == NULL)
	{
		gchar* FOpenErrBox = "ERROR: Could not open the cache.ini file.";
		gnome_error_dialog(FOpenErrBox);
		return;
	}
	lcurfile = "";
	lnum = 0;

	while((buffer = getc(FileList)) != EOF)
	{
		if(buffer == '=')
		{
			buffer = getc(FileList);

			if(lnum > 0)
			{
				gtk_list_store_append(lsCacheList, &iter);
				gtk_list_store_set(lsCacheList, &iter, 1, lcurfile, -1);
			}
			g_free(lcurfile);
			lcurfile = "";
		}
		else
		if(buffer == '\n')
		{
			buffer = getc(FileList);

			if(lnum > 0)
			{
				gtk_list_store_set(lsCacheList, &iter, 0, lcurfile, -1);
			}
			g_free(lcurfile);
			lcurfile = "";
			lnum++;
		}
		lcurfile = g_strdup_printf("%s%c", lcurfile, buffer);
	}
	fclose(FileList);
	g_free(listfile);

	gtk_widget_show_all(CacheWnd);
}

void
on_btnMainUMod_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	get_utinstalldir();
	UModWnd = create_UModManWnd();

	create_model_umodlist(lookup_widget( GTK_WIDGET(UModWnd), "tvUModFiles") );
	gtk_widget_show(UModWnd);
}

void
on_btnMainServer_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	get_utinstalldir();
	ServerWnd = create_DedServerWnd();

	create_model_mutatorlist(lookup_widget( GTK_WIDGET(ServerWnd), "tvServerMutators") );
	gtk_widget_show(ServerWnd);
}

void
on_btnMainCompress_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkFileChooser		*fcCompress;

	GtkFileFilter		*filterall;
	GtkFileFilter		*filteruax;
	GtkFileFilter		*filterukx;
	GtkFileFilter		*filterusx;
	GtkFileFilter		*filterut2;
	GtkFileFilter		*filterutx;
	GtkFileFilter		*filteruz2;
	GtkFileFilter		*filteru;

	get_utinstalldir();
	CompressWnd = create_CompressWnd();
	create_model_compresslist(lookup_widget( GTK_WIDGET(CompressWnd), "tvCompressFiles") );

	fcCompress = GTK_FILE_CHOOSER(lookup_widget(CompressWnd, "gtkfcCompress"));

	filterall = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filterall, "*");
	gtk_file_filter_set_name(filterall, "All Files");

	filteruax = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filteruax, "*.uax");
	gtk_file_filter_set_name(filteruax, "Sounds");

	filterukx = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filterukx, "*.ukx");
	gtk_file_filter_set_name(filterukx, "Animations");

	filterusx = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filterusx, "*.usx");
	gtk_file_filter_set_name(filterusx, "StaticMeshes");

	filterut2 = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filterut2, "*.ut2");
	gtk_file_filter_set_name(filterut2, "Maps");

	filterutx = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filterutx, "*.utx");
	gtk_file_filter_set_name(filterutx, "Textures");

	filteru = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filteru, "*.u");
	gtk_file_filter_set_name(filteru, "System Files");

	filteruz2 = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filteruz2, "*.uz2");
	gtk_file_filter_set_name(filteruz2, "Compressed");

	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fcCompress), filterall);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fcCompress), filteruz2);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fcCompress), filterukx);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fcCompress), filterut2);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fcCompress), filteruax);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fcCompress), filterusx);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fcCompress), filteru);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fcCompress), filterutx);

	gtk_file_chooser_set_show_hidden(GTK_FILE_CHOOSER(fcCompress), TRUE);

	gtk_widget_show(CompressWnd);
}

void
on_btnMainUMark_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	system("umark &");
}

void
on_btnMainConfig_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	get_utinstalldir();
}

void
on_AboutURL_Clicked			(GtkAboutDialog *about,
					const gchar *link,
					gpointer data)
{
	gchar	*browserexec;
	
	browserexec = g_strdup_printf("mozilla %s &", link);
	system(browserexec);
}

void
on_btnMainAbout_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	const gchar	*Link = "http://gunrealtools.sf.net";
	const gchar	*Author[] = {"Jeffrey Bakker", NULL};
/*	GdkPixbuf	*Logo = gdk_pixbuf_new_from_file("gunrealtools/guts.png", NULL);
	GdkPixbuf	*Icon = create_pixbuf("gunrealtools/gunrealtools-icon.png");
*/	GdkPixbuf	*Logo = create_pixbuf("gunrealtools/guts.png");

	gtk_about_dialog_set_url_hook(on_AboutURL_Clicked, (gpointer)Link, NULL);

	AboutWnd = gtk_about_dialog_new();
	gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(AboutWnd), "GUTS (gUnrealTools)");
	gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(AboutWnd), "GUI Tools for UT2004");
	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(AboutWnd), "Copyright (C)2005 Jeffrey Bakker");
	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(AboutWnd), Link);	
	gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(AboutWnd), Author);
/*	gtk_about_dialog_set_icon(GTK_ABOUT_DIALOG(AboutWnd), Icon);
*/	gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(AboutWnd), Logo);

	gtk_dialog_run(GTK_DIALOG(AboutWnd));
	gtk_widget_destroy(AboutWnd);

	g_object_unref(Logo);
}

void
on_btnMainQuit_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_main_quit();
}


/*** Cache Manager Window *****************************************************/
gboolean
foreach_cache_ini (	GtkTreeModel	*model,
			GtkTreePath	*path,
			GtkTreeIter	*iter,
			gpointer	user_data)
{
	gchar	*lcurfile;
	gchar	*cachefile;
	gchar	*uncachefile;
	
	gtk_tree_model_get(model, iter, 0, &uncachefile, -1);
	gtk_tree_model_get(model, iter, 1, &cachefile, -1);

	lcurfile = g_strdup_printf("%s=%s\n", cachefile, uncachefile);
	g_fprintf(user_data, lcurfile);
	g_free(lcurfile);
	
	return FALSE;
}

void
on_btnCacheInstall_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	FILE		*CacheIni;
	gchar		*cache_ini;

	gchar		*fdir;
	gchar		*ftype;
	gchar		*cachefile;
	gchar		*uncachefile;
	GtkListStore	*lsCacheList;
	GtkTreeIter	iter;

	GtkTreeSelection *select;

	lsCacheList = GTK_LIST_STORE(gtk_tree_view_get_model( GTK_TREE_VIEW(lookup_widget(CacheWnd , "tvCacheFiles")) ));

	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(lookup_widget(CacheWnd , "tvCacheFiles")));
	gtk_tree_selection_get_selected(select, (GtkTreeModel **)&lsCacheList, &iter);

	gtk_tree_model_get(GTK_TREE_MODEL(lsCacheList), &iter, 0, &uncachefile, -1);
	gtk_tree_model_get(GTK_TREE_MODEL(lsCacheList), &iter, 1, &cachefile, -1);

	if(strstr(uncachefile, ".ukx") != NULL)
	{
		ftype = "Animations";
	}
	else
	if(strstr(uncachefile, ".ut2") != NULL)
	{
		ftype = "Maps";
	}
	else
	if(strstr(uncachefile, ".uax") != NULL)
	{
		ftype = "Sounds";
	}
	else
	if(strstr(uncachefile, ".usx") != NULL)
	{
		ftype = "StaticMeshes";
	}
	else
	if(strstr(uncachefile, ".utx") != NULL)
	{
		ftype = "Textures";
	}
	else
	if(strstr(uncachefile, ".u") != NULL)
	{
		ftype = "System";
	}
	else	ftype = "";

	cachefile = g_strdup_printf("%s/.ut2004/Cache/%s.uxx", getenv("HOME"), cachefile);
	uncachefile = g_strdup_printf("%s/.ut2004/%s/%s", getenv("HOME"), ftype, uncachefile);
/*
	gnome_error_dialog(cachefile);
	gnome_error_dialog(uncachefile);
*/
	fdir = g_strdup_printf("%s/.ut2004/%s", getenv("HOME"), ftype);
	if(!g_file_test(fdir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
	{
		g_mkdir(fdir, S_IRUSR | S_IWUSR | S_IXUSR);
	}
	if(g_rename(cachefile, uncachefile) == -1)
	{
		return;
	}
	gtk_list_store_remove(GTK_LIST_STORE(lsCacheList), &iter);

	cache_ini = g_strdup_printf("%s/.ut2004/Cache/cache.ini", getenv("HOME"));
	if((CacheIni = fopen(cache_ini, "w")) == NULL)
	{
		g_free(cache_ini);
		return;	
	}
	g_fprintf(CacheIni, "[Cache]\n");

	gtk_tree_model_foreach(GTK_TREE_MODEL(lsCacheList), foreach_cache_ini, CacheIni);
	fclose(CacheIni);
	g_free(cache_ini);
}

void
on_btnCacheDelete_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	FILE		*CacheIni;
	gchar		*cache_ini;

	gchar		*delfile;
	gchar		*cachefile;
	GtkListStore	*lsCacheList;
	GtkTreeIter	iter;

	GtkTreeSelection *select;

	lsCacheList = GTK_LIST_STORE(gtk_tree_view_get_model( GTK_TREE_VIEW(lookup_widget(CacheWnd , "tvCacheFiles")) ));

	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(lookup_widget(CacheWnd , "tvCacheFiles")));
	gtk_tree_selection_get_selected(select, (GtkTreeModel **)&lsCacheList, &iter);

	gtk_tree_model_get(GTK_TREE_MODEL(lsCacheList), &iter, 1, &cachefile, -1);
	delfile = g_strdup_printf("%s/.ut2004/Cache/%s.uxx", getenv("HOME"), cachefile);

	if(g_remove(delfile) == -1)
	{
		g_free(delfile);
		return;
	}
	gtk_list_store_remove(GTK_LIST_STORE(lsCacheList), &iter);

	g_free(delfile);

	cache_ini = g_strdup_printf("%s/.ut2004/Cache/cache.ini", getenv("HOME"));
	if((CacheIni = fopen(cache_ini, "w")) == NULL)
	{
		g_free(cache_ini);
		return;	
	}
	g_fprintf(CacheIni, "[Cache]\n");

	gtk_tree_model_foreach(GTK_TREE_MODEL(lsCacheList), foreach_cache_ini, CacheIni);
	fclose(CacheIni);
	g_free(cache_ini);
}

void
on_btnCacheClear_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	FILE		*CacheIni;
	gchar		*cache_ini;
	gchar		*cachedir;
	
	GtkListStore	*lsCacheList;

	lsCacheList = GTK_LIST_STORE(gtk_tree_view_get_model( GTK_TREE_VIEW(lookup_widget(CacheWnd , "tvCacheFiles")) ));

	cachedir = g_strdup_printf("rm -f %s/.ut2004/Cache/*.uxx", getenv("HOME"));
	if(system(cachedir) == -1)
	{
		return;
	}
	gtk_list_store_clear(GTK_LIST_STORE(lsCacheList));

	cache_ini = g_strdup_printf("%s/.ut2004/Cache/cache.ini", getenv("HOME"));
	if((CacheIni = fopen(cache_ini, "w")) == NULL)
	{
		g_free(cache_ini);
		return;
	}
	g_fprintf(CacheIni, "[Cache]\n");

	gtk_tree_model_foreach(GTK_TREE_MODEL(lsCacheList), foreach_cache_ini, CacheIni);
	fclose(CacheIni);
	g_free(cache_ini);
}

void
on_btnCacheClose_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(CacheWnd);
}

void
on_tvCacheFiles_cursor_changed         (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	struct	stat	filestat;

	gchar		*ftype;
	gchar		*finfo;
	gchar		*fname;
	gchar		*fhash;
	gchar		*strTmp;
	GtkTreeView	*tvCacheFiles;
	GtkListStore	*lsCacheList;
	GtkTreeIter	iter;

	GtkTreeSelection *select;
	GtkTextBuffer*	TxtBuf;

	TxtBuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(lookup_widget(CacheWnd, "txtviewCacheInfo")));
	
	tvCacheFiles = GTK_TREE_VIEW(lookup_widget(CacheWnd , "tvCacheFiles"));
	lsCacheList = GTK_LIST_STORE(gtk_tree_view_get_model(tvCacheFiles));

	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tvCacheFiles));
	gtk_tree_selection_get_selected(select, (GtkTreeModel **)&lsCacheList, &iter);

	gtk_tree_model_get(GTK_TREE_MODEL(lsCacheList), &iter, 0, &strTmp, -1);
	gtk_tree_model_get(GTK_TREE_MODEL(lsCacheList), &iter, 1, &fhash, -1);

	fname = g_strdup_printf("%s/.ut2004/Cache/%s.uxx", getenv("HOME"), fhash);

	if(g_stat(fname, &filestat) != 0)
	{
/*		gchar* FSizeErrBox = "ERROR: Could not read the file size.";
		umark_message_dialog(FSizeErrBox);
*/		return;
	}

	if(strstr(strTmp, ".ukx") != NULL)
	{
		ftype = "Animations";
	}
	else
	if(strstr(strTmp, ".ut2") != NULL)
	{
		ftype = "Maps";
	}
	else
	if(strstr(strTmp, ".uax") != NULL)
	{
		ftype = "Sounds";
	}
	else
	if(strstr(strTmp, ".usx") != NULL)
	{
		ftype = "StaticMeshes";
	}
	else
	if(strstr(strTmp, ".utx") != NULL)
	{
		ftype = "Textures";
	}
	else
	if(strstr(strTmp, ".u") != NULL)
	{
		ftype = "System";
	}
	else	ftype = "";

	finfo = g_strdup_printf("Filesize : %d bytes\nInstall to: ~/.ut2004/%s", (int)filestat.st_size, ftype);
	gtk_text_buffer_set_text(TxtBuf, finfo, strlen(finfo));
}


/*** Dedicated Server Setup Window ********************************************/
gboolean
foreach_mutator(	GtkTreeModel	*model,
			GtkTreePath	*path,
			GtkTreeIter	*iter,
			gpointer	user_data)
{
	gchar	*lcurmut;
	gchar	*mutator;

	/*
	Air Control - UnrealGame.MutMovementModifier *
	Arena - XWeapons.MutArena
	Berserk - UnrealGame.MutBerserk
	Big Head - UnrealGame.MutBigHead
	Big Wheels - Onslaught.MutBigWheels *
	Bonus Combos - BonusPack.MutCrateCombo,Bonuspack.MutPintSizeCombo
	Delayed Spawn - UnrealGame.MutDelayedSpawn
	Floating Corpses - XGame.MutHeliumCorpses
	Game Speed - UnrealGame.MutGameSpeed *
	InstaGib - XGame.MutInstaGib
	Lightning Guns - UTClassic.MutUseLightning *
	Lightweight Vehicles - Onslaught.MutLightweightVehicles *
	Low Gravity - UnrealGame.MutLowGrav
	No Adrenaline - XGame.MutNoAdrenaline
	No Super Weapons - XWeapons.MutNoSuperWeapon
	Onslaught Weapons - Onslaught.MutOnslaughtWeapons *
	Quad Jump - XGame.MutQuadJump
	Regeneration - XGame.MutRegen
	Slow Motion Corpses - XGame.MutSlomoDeath
	Snipe Rifles - UTClassic.MutUseSniper *
	Species Statistics - XGame.MutSpeciesStats
	Stunt Vehicles - OnslaughtFull.MutVehicleArena *
	UDamage Reward - XGame.MutUDamageReward
	UT Classic - UTClassic.MutUTClassic *
	UT2003 Style - XGame.MutFastWeapSwitch
	UTV2004S - UTV2004s.utvMutator
	Vampire - XGame.MutVampire
	Vehicle Arena - OnslaughtFull.MutVehicleArena
	Vehicle Pickups -
	Zoom InstaGib - XGame.MutZoomInstagib
	
	*/

	gtk_tree_model_get(model, iter, 0, &lcurmut, -1);

	if(strstr(lcurmut, "Air Control") != NULL)
	{
		mutator = "UnrealGame.MutMovementModifier";
	}
	else
	if(strstr(lcurmut, "Arena") != NULL)
	{
		mutator = "XWeapons.MutArena";
	}
	else
	if(strstr(lcurmut, "Berserk") != NULL)
	{
		mutator = "UnrealGame.MutBerserk";
	}
	else
	if(strstr(lcurmut, "Big Head") != NULL)
	{
		mutator = "UnrealGame.MutBigHead";
	}
	else
	if(strstr(lcurmut, "Big Wheels") != NULL)
	{
		mutator = "Onslaught.MutBigWheels";
	}
	else
	if(strstr(lcurmut, "Bonus Combos") != NULL)
	{
		mutator = "BonusPack.MutCrateCombo,Bonuspack.MutPintSizeCombo";
	}
	else
	if(strstr(lcurmut, "Delayed Spawn") != NULL)
	{
		mutator = "UnrealGame.MutDelayedSpawn";
	}
	else
	if(strstr(lcurmut, "Floating Corpses") != NULL)
	{
		mutator = "XGame.MutHeliumCorpses";
	}
	else
	if(strstr(lcurmut, "Game Speed") != NULL)
	{
		mutator = "UnrealGame.MutGameSpeed";
	}
	else
	if(strstr(lcurmut, "InstaGib") != NULL)
	{
		mutator = "XGame.MutInstaGib";
	}
	else
	if(strstr(lcurmut, "Lightning Guns") != NULL)
	{
		mutator = "UTClassic.MutUseLightning";
	}
	else
	if(strstr(lcurmut, "Lightweight Vehicles") != NULL)
	{
		mutator = "Onslaught.MutLightweightVehicles";
	}
	else
	if(strstr(lcurmut, "Low Gravity") != NULL)
	{
		mutator = "UnrealGame.MutLowGrav";
	}
	else
	if(strstr(lcurmut, "No Adrenaline") != NULL)
	{
		mutator = "XGame.MutNoAdrenaline";
	}
	else
	if(strstr(lcurmut, "No Super Weapons") != NULL)
	{
		mutator = "XWeapons.MutNoSuperWeapon";
	}
	else
	if(strstr(lcurmut, "Onslaught Weapons") != NULL)
	{
		mutator = "Onslaught.MutOnslaughtWeapons";
	}
	else
	if(strstr(lcurmut, "Quad Jump") != NULL)
	{
		mutator = "XGame.MutQuadJump";
	}
	else
	if(strstr(lcurmut, "Regeneration") != NULL)
	{
		mutator = "XGame.MutRegen";
	}
	else
	if(strstr(lcurmut, "Slow Motion Corpses") != NULL)
	{
		mutator = "XGame.MutSlomoDeath";
	}
	else
	if(strstr(lcurmut, "Snipe Rifles") != NULL)
	{
		mutator = "UTClassic.MutUseSniper";
	}
	else
	if(strstr(lcurmut, "Species Statistics") != NULL)
	{
		mutator = "XGame.MutSpeciesStats";
	}
	else
	if(strstr(lcurmut, "Stunt Vehicles") != NULL)
	{
		mutator = "OnslaughtFull.MutVehicleArena";
	}
	else
	if(strstr(lcurmut, "UDamage Reward") != NULL)
	{
		mutator = "XGame.MutUDamageReward";
	}
	else
	if(strstr(lcurmut, "UT Classic") != NULL)
	{
		mutator = "UTClassic.MutUTClassic";
	}
	else
	if(strstr(lcurmut, "UT2003 Style") != NULL)
	{
		mutator = "XGame.MutFastWeapSwitch";
	}
	else
	if(strstr(lcurmut, "UTV2004S") != NULL)
	{
		mutator = "UTV2004s.utvMutator";
	}
	else
	if(strstr(lcurmut, "Vampire") != NULL)
	{
		mutator = "XGame.MutVampire";
	}
	else
	if(strstr(lcurmut, "Vehicle Arena") != NULL)
	{
		mutator = "OnslaughtFull.MutVehicleArena";
	}
	else
	if(strstr(lcurmut, "Zoom InstaGib") != NULL)
	{
		mutator = "XGame.MutZoomInstagib";
	}
	else
	{
		mutator = "";
	}


	fprintf(user_data, "%s,", mutator);

	g_free(lcurmut);

	return FALSE;
}


void
on_btnServerStart_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar	*CurMut = "";
	gchar	*Mutators = "";
	gchar	*ServerCmd = "";
	gchar	*GameOptions = "";
	gchar	*ExtraOptions = "";


	GtkListStore	*lsMutList;
	GtkTreeModel	*tmMutList;
	GtkTreePath	*tpMutList;

	tmMutList = gtk_tree_view_get_model( GTK_TREE_VIEW(lookup_widget(ServerWnd, "tvServerMutators")) );

	FILE	*GutsServer;
	
	GutsServer = fopen(g_strconcat(getenv("HOME"), "/gutsserver.sh", NULL), "w");
	if(!GutsServer)
	{
		return;
	}

	/*
	GoalScore integer :)
	TimeLimit integer :)
	WeaponStay boolean :)
	bBalanceTeams boolean :)
	AllowBehindview boolean :)
	AllowThrowing boolean :)
	Translocator boolean :)
	ForceRespawn?
	bPlayerMustBeReady boolean :)
	
	Email?
	AdminName string :)
	AdminPassword string :)
	GamePassword string :)
	Name?
	MOTD?
	MaxPlayers integer :)
	MaxSpectators integer :)
	MinPlayers integer :)
	GameStats boolean :)
	
	*/

	GameOptions = g_strdup_printf("?GoalScore=%d", GoalScore);
	GameOptions = g_strdup_printf("%s?TimeLimit=%d", GameOptions, TimeLimit);

	if(WeaponStay)
	{
		GameOptions = g_strconcat(GameOptions, "?WeaponStay=1", NULL);
	}
	if(UseTeamSkins) /* BalanceTeams */
	{
		GameOptions = g_strconcat(GameOptions, "?bBalanceTeams=1", NULL);
	}
	if(BehindView)
	{
		GameOptions = g_strconcat(GameOptions, "?AllowBehindview=1", NULL);
	}
	if(WeaponThrow)
	{
		GameOptions = g_strconcat(GameOptions, "?AllowThrowing=1", NULL);
	}
	if(Translocator)
	{
		GameOptions = g_strconcat(GameOptions, "?Translocator=1", NULL);
	}
	if(MustBeReady)
	{
		GameOptions = g_strconcat(GameOptions, "?bPlayerMustBeReady=1", NULL);
	}
	
	GameOptions = g_strdup_printf("%s?AdminName=%s", GameOptions, AdminName);
	GameOptions = g_strdup_printf("%s?AdminPassword=%s", GameOptions, AdminPass);

	GameOptions = g_strdup_printf("%s?MaxPlayers=%d", GameOptions, MaxPlayers);
	GameOptions = g_strdup_printf("%s?MaxSpectators=%d", GameOptions, MaxSpectators);
	GameOptions = g_strdup_printf("%s?MinPlayers=%d", GameOptions, MinNetPlayers);

	if(strcmp("",GamePass) != 0)
	{
		GameOptions = g_strdup_printf("%s?GamePassword=%s", GameOptions, GamePass);
	}
	if(StatsLogging)
	{
		GameOptions = g_strconcat(GameOptions, "?GameStats=1", NULL);	
	}

/*	ServerCmd = g_strdup_printf("%s/System/ucc-bin server %s?Game=%s%s%s %s -nohomedir",
			UTDir, map, GameType, Mutators, GameOptions, ExtraOptions);
	
	system(ServerCmd);
*/
/*
	ServerCmd = g_strdup_printf("%s/System/ucc-bin server %s?Game=%s%s%s %s ini=%s/System/UT2004.ini -nohomedir",
			UTDir, map, GameType, Mutators, GameOptions, ExtraOptions, UTDir);
	
	GError *error;
	g_spawn_command_line_async(ServerCmd, &error);
*/
/*
	ServerCmd = g_strdup_printf("xterm -e %s/System/ucc-bin server %s?Game=%s%s%s %s",
			UTDir, map, GameType, Mutators, GameOptions, ExtraOptions);
	
	gnome_execute_terminal_shell(g_strconcat(UTDir, "/System"), ServerCmd);
*/

	fprintf(GutsServer, "#!/bin/bash\n\ncd %s/System\n\nwhile true\ndo\n", UTDir);
	fprintf(GutsServer, "%s/System/ucc-bin server %s?Game=%s", UTDir, map, GameType);	
	fprintf(GutsServer, "?Mutator=");
	gtk_tree_model_foreach(GTK_TREE_MODEL(tmMutList), foreach_mutator, GutsServer);
	fprintf(GutsServer, "%s %s\ndone\n", GameOptions, ExtraOptions);

/*	g_printf("%s", ServerCmd);
*/
	close(GutsServer);

/*	g_free(ServerCmd);
*/
}

void
on_btnServerClose_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(ServerWnd);
}

void
on_cmbServerGametype_changed           (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	gint	type;
	gchar	buffer;
	gchar	*lcurmap;
	gchar	*MapPath;
	FILE	*MapList;
	
	GtkComboBox	*cbMaps;
	GtkTreeModel	*cbTmod;
	cbMaps = GTK_COMBO_BOX(lookup_widget(ServerWnd , "cmbServerMap"));
	cbTmod = GTK_TREE_MODEL(gtk_combo_box_get_model(cbMaps));

/*	GameType = (gchar *)gtk_combo_box_get_active_text(combobox);
*/

/*

GameType - MapFilter GameClass

Assault - AS UT2k4Assault.ASGameInfo
Bombing Run - BR xGame.xBombingRun
Capture the Flag - CTF xGame.xCTFGame
Double Domination - DOM xGame.xDoubleDom
DeathMatch - DM xGame.xDeathMatch
Invasion - DM SkaarjPack.Invasion
Last Man Standing - DM BonusPack.xLastManStandingGame
Mutant - DM BonusPack.xMutantGame
Onslaught - ONS Onslaught.ONSOnslaughtGame
Team DeathMatch - DM xGame.xTeamGame
Vehicle CTF - xGame.xVehicleCTFGame

*/

	type = gtk_combo_box_get_active(combobox);

	switch(type)
	{
		case (0): MapFilter = "AS"; GameType = "UT2k4Assault.ASGameInfo"; break;
		case (1): MapFilter = "BR"; GameType = "xGame.xBombingRun"; break;
		case (2): MapFilter = "CTF"; GameType = "xGame.xCTFGame"; break;
		case (3): MapFilter = "DM"; GameType = "xGame.xDoubleDom"; break;
		case (4): MapFilter = "DOM"; GameType = "xGame.xDeathMatch"; break;
		case (5): MapFilter = "CTF"; GameType = "xGame.xCTFGame"; break;
		case (6): MapFilter = "DM"; GameType = "SkaarjPack.Invasion"; break;
		case (7): MapFilter = "DM"; GameType = "BonusPack.xLastManStandingGame"; break;
		case (8): MapFilter = "DM"; GameType = "BonusPack.xMutantGame"; break;
		case (9): MapFilter = "ONS"; GameType = "Onslaught.ONSOnslaughtGame"; break;
		case (10): MapFilter = "DM"; GameType = "xGame.xTeamGame"; break;
		case (11): MapFilter = "CTF"; GameType = "xGame.xVehicleCTFGame"; break;
	}

	gtk_list_store_clear(GTK_LIST_STORE(cbTmod));

	MapPath = g_strdup_printf("ls -1 %s/Maps | grep %s >%s/.UTMAP_LST", UTDir, MapFilter, getenv("HOME"));
	system(MapPath);
	g_free(MapPath);
	MapPath = g_strdup_printf("%s/.UTMAP_LST", getenv("HOME"));
	MapList = fopen(MapPath, "r");

	lcurmap = "";

	while((buffer = getc(MapList)) != EOF)
	{
		if(buffer != '\n')
		{
			lcurmap = g_strdup_printf("%s%c", lcurmap, buffer); /* TODO: find a better way to append */
		}
		else
		{
			gtk_combo_box_append_text(GTK_COMBO_BOX(cbMaps), lcurmap);
			g_free(lcurmap);
			lcurmap = "";
		}
	}
	fclose(MapList);
	remove(MapPath);
	g_free(MapPath);
}

void
on_cmbServerMap_changed                (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	map = gtk_combo_box_get_active_text(combobox);
}

void
on_spnServerScore_changed              (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GoalScore = atoi(gtk_entry_get_text(GTK_ENTRY(editable)));
}

void
on_spnServerTimelimit_changed          (GtkEditable     *editable,
                                        gpointer         user_data)
{
	TimeLimit = atoi(gtk_entry_get_text(GTK_ENTRY(editable)));
}

void
on_chkServerWeapStay_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	WeaponStay = !WeaponStay;
}

void
on_chkServerTeamSkins_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	UseTeamSkins = !UseTeamSkins;
}

void
on_chkServerBehindView_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	BehindView = !BehindView;
}

void
on_chkServerWeapThrow_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	WeaponThrow = !WeaponThrow;
}

void
on_chkServerTranslocator_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	Translocator = !Translocator;
}

void
on_chkServerForceRespawn_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	ForceRespawn = !ForceRespawn;
}

void
on_chkServerMustBeReady_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	MustBeReady = !MustBeReady;
}

void
on_entServerAdminEmail_changed         (GtkEditable     *editable,
                                        gpointer         user_data)
{
	AdminEmail = (gchar *)gtk_entry_get_text(GTK_ENTRY(editable));
}

void
on_entServerAdminName_changed          (GtkEditable     *editable,
                                        gpointer         user_data)
{
	AdminName = (gchar *)gtk_entry_get_text(GTK_ENTRY(editable));
}

void
on_antServerAdminPass_changed          (GtkEditable     *editable,
                                        gpointer         user_data)
{
	AdminPass = (gchar *)gtk_entry_get_text(GTK_ENTRY(editable));
}

void
on_entServerGamePass_changed           (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GamePass = (gchar *)gtk_entry_get_text(GTK_ENTRY(editable));
}

void
on_entServerName_changed               (GtkEditable     *editable,
                                        gpointer         user_data)
{
	ServerName = (gchar *)gtk_entry_get_text(GTK_ENTRY(editable));
}

void
on_entServerMOTD_changed               (GtkEditable     *editable,
                                        gpointer         user_data)
{
	ServerMOTD = (gchar *)gtk_entry_get_text(GTK_ENTRY(editable));
}

void
on_spnMaxSpectators_changed            (GtkEditable     *editable,
                                        gpointer         user_data)
{
	MaxSpectators = atoi(gtk_entry_get_text(GTK_ENTRY(editable)));
}

void
on_spnServerMinNet_changed             (GtkEditable     *editable,
                                        gpointer         user_data)
{
	MinNetPlayers = atoi(gtk_entry_get_text(GTK_ENTRY(editable)));
}

void
on_spnServerMaxPlayers_changed         (GtkEditable     *editable,
                                        gpointer         user_data)
{
	MaxPlayers = atoi(gtk_entry_get_text(GTK_ENTRY(editable)));
}

void
on_chkServerWorldStats_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	StatsLogging = !StatsLogging;
}

void
on_cmbServerMutators_changed           (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	mut = (gchar *)gtk_combo_box_get_active_text(combobox);
}

void
on_btnServerAddMut_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkTreeView	*tvMutators;
	GtkListStore	*lsMutList;
	GtkTreeIter	iter;

	tvMutators = GTK_TREE_VIEW(lookup_widget(ServerWnd , "tvServerMutators"));
	lsMutList = GTK_LIST_STORE(gtk_tree_view_get_model(tvMutators));

	gtk_list_store_append(GTK_LIST_STORE(lsMutList), &iter);
	gtk_list_store_set(GTK_LIST_STORE(lsMutList), &iter, 0, mut, -1);
}

void
on_btnServerRemoveMut_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkTreeView	*tvMutators;
	GtkListStore	*lsMutList;
	GtkTreeIter	iter;

	GtkTreeSelection *select;

	tvMutators = GTK_TREE_VIEW(lookup_widget(ServerWnd , "tvServerMutators"));
	lsMutList = GTK_LIST_STORE(gtk_tree_view_get_model(tvMutators));

	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tvMutators));
	gtk_tree_selection_get_selected(select, (GtkTreeModel **)&lsMutList, &iter);
	gtk_list_store_remove(GTK_LIST_STORE(lsMutList), &iter);
}

void
on_btnServerClearMuts_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkListStore	*lsMutList;

	lsMutList = GTK_LIST_STORE(gtk_tree_view_get_model( GTK_TREE_VIEW(lookup_widget(ServerWnd , "tvServerMutators")) ));
	gtk_list_store_clear(GTK_LIST_STORE(lsMutList));
}


/*** UMod Unpacker Window *****************************************************/
static gboolean
progressbar_timeout(gpointer data)
{
	gtk_progress_bar_pulse(GTK_PROGRESS_BAR(data));
	
	return TRUE;
}

gpointer
umod_unpack_execute			(gpointer data)
{
	gboolean finished = FALSE;

	gchar	buffer;
	gchar	*cfile;
	gchar	*cwdir;
	gchar	*sydir;

	FILE	*fpipe;
	gchar	*UModExec;
 
	GtkTreeIter	iter;
	GtkListStore	*lsMutList;

	lsMutList = GTK_LIST_STORE(gtk_tree_view_get_model( GTK_TREE_VIEW(lookup_widget(UModWnd , "tvUModFiles")) ));
	gtk_list_store_clear(GTK_LIST_STORE(lsMutList));

	/* FIXME: commandlets are not found unless ucc-bin is ran from ut2004/System dir*/
	sydir = g_strdup_printf("%s/System", UTDir);

	chdir(sydir);
	g_free(sydir);

	UModExec = g_strdup_printf("%s/System/ucc-bin umodunpack -x %s", UTDir, UModFile);

	if((fpipe = (FILE*)popen(UModExec, "r")) == NULL)
	{
		return NULL;
	}
	cfile = "";

	while(!feof(fpipe) || !finished)
	{
		buffer = getc(fpipe);

		if(buffer == EOF)
		{
			finished = TRUE;
		}
		else
		if(buffer == '\n')
		{
			gtk_list_store_append(lsMutList, &iter);
			gtk_list_store_set(lsMutList, &iter, 0, cfile, -1);
			g_free(cfile);
			cfile = "";

			buffer = getc(fpipe);
		}
		cfile = g_strdup_printf("%s%c", cfile, buffer);
	}
	pclose(fpipe);
	g_free(UModExec);
	
	g_source_remove(GPOINTER_TO_INT(g_object_get_data(G_OBJECT(data), "source_id")));
	gtk_widget_hide(data);
	gtk_widget_destroy(data);

/*	cwdir = getcwd(cwdir, 255);
*/
/*	chdir(cwdir);
*/
	return NULL;
}

gpointer
umod_unpack_list			(gpointer data)
{
	gboolean finished = FALSE;

	gchar	buffer;
	gchar	*cfile;
	gchar	*cwdir;
	gchar	*sydir;

	FILE	*fpipe;
	gchar	*UModExec;
 
	GtkTreeIter	iter;
	GtkListStore	*lsMutList;

	lsMutList = GTK_LIST_STORE(gtk_tree_view_get_model( GTK_TREE_VIEW(lookup_widget(UModWnd , "tvUModFiles")) ));
	gtk_list_store_clear(GTK_LIST_STORE(lsMutList));

	/* FIXME: commandlets are not found unless ucc-bin is ran from ut2004/System dir*/
	sydir = g_strdup_printf("%s/System", UTDir);

	chdir(sydir);
	g_free(sydir);
	
	UModExec = g_strdup_printf("%s/System/ucc-bin umodunpack -l %s", UTDir, UModFile);

	if((fpipe = (FILE*)popen(UModExec, "r")) == NULL)
	{
		return NULL;
	}
	cfile = "";

	while(!feof(fpipe) || !finished)
	{
		buffer = getc(fpipe);

		if(buffer == EOF)
		{
			finished = TRUE;
		}
		else
		if(buffer == '\n')
		{
			gtk_list_store_append(lsMutList, &iter);
			gtk_list_store_set(lsMutList, &iter, 0, cfile, -1);
			g_free(cfile);
			cfile = "";

			buffer = getc(fpipe);
		}
		cfile = g_strdup_printf("%s%c", cfile, buffer);
	}
	pclose(fpipe);
	g_free(UModExec);

	g_source_remove(GPOINTER_TO_INT(g_object_get_data(G_OBJECT(data), "source_id")));
	gtk_widget_hide(data);
	gtk_widget_destroy(data);

/*	cwdir = getcwd(cwdir, 255);
*/
/*	chdir(cwdir);
*/
	return NULL;
}

void
on_entUModFile_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	UModFile = (gchar *)gtk_entry_get_text(GTK_ENTRY(editable));

/*	umod_unpack_execute('l');
*/
	gint 		timer;
	GtkProgressBar	*pBar;
	GtkWidget	*pWnd;
	
	pWnd = create_ProgressWnd();
	gtk_widget_show(pWnd);

	pBar = GTK_PROGRESS_BAR(lookup_widget(pWnd, "pbarProgress"));

	timer = g_timeout_add(100, progressbar_timeout, pBar);
	g_object_set_data(G_OBJECT(pWnd), "source_id", GINT_TO_POINTER(timer));

	g_thread_create(umod_unpack_list, pWnd, FALSE, NULL);	
/*	g_source_remove(GPOINTER_TO_INT(g_object_get_data(G_OBJECT(pBar), "source_id")));
*/
}

void
on_btnUModBrowse_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
/*	GtkWidget	*ConfDlg;
*/	
	GtkFileFilter	*filter3;
	GtkFileFilter	*filter4;
	GtkFileFilter	*filtera;
	GtkWidget	*OpenDlg;
	gchar		*browse_umod;

	OpenDlg = gtk_file_chooser_dialog_new(
		"Open UMod File", NULL,
		GTK_FILE_CHOOSER_ACTION_OPEN,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
		NULL);

	gtk_file_chooser_set_show_hidden(GTK_FILE_CHOOSER(OpenDlg), TRUE);

	filter3 = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter3, "*.ut2mod");
	gtk_file_filter_set_name(filter3, "UT2003 Mod");

	filter4 = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter4, "*.ut4mod");
	gtk_file_filter_set_name(filter4, "UT2004 Mod");

	filtera = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filtera, "*");
	gtk_file_filter_set_name(filtera, "All Files");

	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(OpenDlg), filter4);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(OpenDlg), filter3);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(OpenDlg), filtera);
	
/*	ConfDlg = GTK_WIDGET( gtk_widget_get_toplevel(GTK_WIDGET(button)) );
*/
	if(gtk_dialog_run(GTK_DIALOG(OpenDlg)) == GTK_RESPONSE_ACCEPT)
	{
		browse_umod = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(OpenDlg));

		gtk_entry_set_text(GTK_ENTRY( lookup_widget(UModWnd, "entUModFile") ), browse_umod);
/*		UModFile = g_strdup(browse_umod);
*/
		g_free(browse_umod);
	}
	gtk_widget_destroy(OpenDlg);
}

void
on_btnUModUnpack_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	if(UModFile == "")
	{
		return;	
	}

	gint 		timer;
	GtkProgressBar	*pBar;
	GtkWidget	*pWnd;
	
	pWnd = create_ProgressWnd();
	gtk_widget_show(pWnd);

	pBar = GTK_PROGRESS_BAR(lookup_widget(pWnd, "pbarProgress"));

	timer = g_timeout_add(100, progressbar_timeout, pBar);
	g_object_set_data(G_OBJECT(pWnd), "source_id", GINT_TO_POINTER(timer));

/*	umod_unpack_execute('x');
*/	
	g_thread_create(umod_unpack_execute, pWnd, FALSE, NULL);
/*	g_source_remove(GPOINTER_TO_INT(g_object_get_data(G_OBJECT(pBar), "source_id")));
*/
}

void
on_btnUModClose_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(UModWnd);
}


/*** uz2 Compressor Window ****************************************************/
gboolean
foreach_compress_file(	GtkTreeModel	*model,
			GtkTreePath	*path,
			GtkTreeIter	*iter,
			gpointer	user_data)
{
	struct	stat filestat;
	gchar	*uz2command;
	gchar	*movcommand;
	gchar	*lcurfile;
	gchar	*fresult;

	gtk_tree_model_get(model, iter, 0, &lcurfile, -1);

/*	uz2command = g_strdup_printf("./ucc-bin %s %s", (gchar*)user_data, lcurfile);
*/
	uz2command = g_strdup_printf("%s/System/ucc-bin %s %s", UTDir, (gchar*)user_data, lcurfile);
	system(uz2command);
	g_free(uz2command);

	if(strcmp(user_data, "compress") == 0)
	{
		fresult = g_strdup_printf("%s.uz2", lcurfile);
		movcommand = g_strdup_printf("mv %s/.ut2004/System%s.uz2 %s.uz2", getenv("HOME"), lcurfile, lcurfile);
	}
	else
	{
		fresult = g_strndup(lcurfile, strlen(lcurfile) -4);
		movcommand = g_strdup_printf("mv %s/.ut2004/System%s %s", getenv("HOME"), fresult, fresult);	
	}
	system(movcommand);

	if(g_stat(fresult, &filestat) != 0)
	{
		gchar* FSizeErrBox = "ERROR: Could not read the file size.";
		
		g_free(movcommand);
		g_free(lcurfile);
		g_free(fresult);

		return FALSE;
	}
	gtk_list_store_set(GTK_LIST_STORE(model), iter, 0, fresult, -1);
	gtk_list_store_set(GTK_LIST_STORE(model), iter, 1, (gulong)filestat.st_size, -1);
	g_free(fresult);

	g_free(movcommand);
	g_free(lcurfile);

	return FALSE;
}

void
on_btnCompressComp_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar		*sydir;
	GtkTreeModel	*tmCompress;
	tmCompress = gtk_tree_view_get_model( GTK_TREE_VIEW(lookup_widget(CompressWnd, "tvCompressFiles")) );

	sydir = g_strdup_printf("%s/System", UTDir);

	chdir(sydir);
	g_free(sydir);

	gtk_tree_model_foreach(GTK_TREE_MODEL(tmCompress), foreach_compress_file, "compress");
}

void
on_btnCompressDecomp_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar		*sydir;
	GtkTreeModel	*tmCompress;
	tmCompress = gtk_tree_view_get_model( GTK_TREE_VIEW(lookup_widget(CompressWnd, "tvCompressFiles")) );

	sydir = g_strdup_printf("%s/System", UTDir);

	chdir(sydir);
	g_free(sydir);

	gtk_tree_model_foreach(GTK_TREE_MODEL(tmCompress), foreach_compress_file, "decompress");
}

void
on_btnCompressClose_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(CompressWnd);
}

void
on_btnCompressAdd_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkFileChooser	*fcCompress;
	GtkTreeView	*tvCompress;
	GtkListStore	*lsCompress;
	GtkTreeIter	iter;
	gchar		*tfile;
	struct	stat	filestat;
	
	fcCompress = GTK_FILE_CHOOSER(lookup_widget(CompressWnd, "gtkfcCompress"));
	tvCompress = GTK_TREE_VIEW(lookup_widget(CompressWnd , "tvCompressFiles"));
	lsCompress = GTK_LIST_STORE(gtk_tree_view_get_model(tvCompress));

	tfile = gtk_file_chooser_get_filename(fcCompress);
	
	if(tfile == NULL)
	{
		return;
	}
	if(g_stat(tfile, &filestat) != 0)
	{
		gchar* FSizeErrBox = "ERROR: Could not read the file size.";
/*		umark_message_dialog(FSizeErrBox);
*/		return;
	}
	gtk_list_store_append(GTK_LIST_STORE(lsCompress), &iter);
	gtk_list_store_set(GTK_LIST_STORE(lsCompress), &iter, 0, tfile, -1);
	gtk_list_store_set(GTK_LIST_STORE(lsCompress), &iter, 1, (gulong)filestat.st_size, -1);
}

void
on_btnCompressRemove_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkListStore	*lsCompress;
	GtkTreeIter	iter;

	GtkTreeSelection *select;

	lsCompress = GTK_LIST_STORE(gtk_tree_view_get_model( GTK_TREE_VIEW(lookup_widget(CompressWnd , "tvCompressFiles")) ));

	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(lookup_widget(CompressWnd , "tvCompressFiles")));
	gtk_tree_selection_get_selected(select, (GtkTreeModel **)&lsCompress, &iter);
	gtk_list_store_remove(GTK_LIST_STORE(lsCompress), &iter);
}

void
on_btnCompressClear_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkListStore	*lsCompress;

	lsCompress = GTK_LIST_STORE(gtk_tree_view_get_model( GTK_TREE_VIEW(lookup_widget(CompressWnd , "tvCompressFiles")) ));
	gtk_list_store_clear(GTK_LIST_STORE(lsCompress));
}
